function [prog, M, N] = sosjointpos_mat_ker(prog,n,d1,d2,var1,var2,I)
%
% SOSJOINTPOS_MAT_KER(prog,n,d1,d2,var1,var2,I) declares the multiplier $M(s)$ and
% kernel part $N(s,t)$ of a positive multiplier+integral operator. The
% multiplier and kernel have the form
%
% M(s) = Z(s)^T Q_1 Z(s)
% N(s,t) = Z(s)^T Q_{12} Z(s,t) + Z(t,s)^T Q_{21} Z(t) + \int_{I(1)}^{I(2)}
% Z(ss,s)^T Q_{22} Z(ss,t) dss
% Q = [ Q_{11}  Q_{12} ]
%     [ Q_{21}  Q_{22} ] >0
%
% 
% where Z(x)= Z_d1(x) \otimes I_n and Z_d(x) is the vector of monomials in
% variables x of degree d1 or less. Z(x,y) = Z_{d1}(x) \otimes Z_{d2}(x)
% \otimes I_n. If the application is stability of time-delay systems, d1
% will eventually be more or less the degree of the multiplier and d2 more
% or less the degree of the kernel function.
% we recommend setting d2 = d1 to keep the polynomial degrees balanced.

% INPUTS 
%   prog: SOS program to modify.
%   n: dimension of the resulting matrix
%   d1: degree of the multiplier (sort of)
%   d2: degree of the kernel
%   var1: Single pvar corresponding to s in the description
%   var2: Single pvar corresponding to t in the description
%   I = [l u] interval of integration
%
% OUTPUT 
%   N1: subdiagonal function
%   N2: superdiagonal superdiagonal
%
% NOTES:
% Distributed with DelayTOOLS
% Compatable with MULTIPOLY and SOSTOOLS as of June 2013
% For support, contact M. Peet, Arizona State University at mpeet@asu.edu

if ~isvalid(var1) || ~isvalid(var2)
  error(['vars must be a polynomial variable']);
end
dum=polynomial(1,1,{'ss'},[1 1]);

% I'm going to construct Z manually so I know where everything is
nZth=d1+1;
Z1degmat = [0:d1]';
Z1coeff = speye(nZth);
Z1varname = var1.varname;
Z1matdim = [nZth 1];
ZZZth=polynomial(Z1coeff,Z1degmat,Z1varname,Z1matdim);


% There are two configurations for the degree d2. Probably the most logical one is 
% that for TDS applications, 2d1 ends up as the degree of the multiplier and
% 2d2 as the degree of the kernel. This is the following:

Z2degmat = [repmat([0:d1]',d2+1,1),vec(repmat(0:d2,d1+1,1))];
nZthksi=length(Z2degmat);
Z2coeff = speye(nZthksi);
Z2varname = [var1.varname; var2.varname];
Z2matdim = [nZthksi 1];
ZZZthksi=polynomial(Z2coeff,Z2degmat,Z2varname,Z2matdim);

% For other applications, however, we may want 2d2 to represent the
% internal variable of integration, in which case we use the following:
% 
% Z2degmat = [repmat([0:d2]',d1+1,1),vec(repmat(0:d1,d2+1,1))];
% nZthksi=length(Z2degmat);
% Z2coeff = speye(nZthksi);
% Z2varname = [var1.varname; var2.varname];
% Z2matdim = [nZthksi 1];
% ZZZthksi=polynomial(Z2coeff,Z2degmat,Z2varname,Z2matdim);


nBZ1=n*nZth;
nBZ2=n*nZthksi;
[prog,LLL]=sosposmatr(prog,nBZ1+nBZ2);


% Lets start by just assuming LLL has been partitioned
% In this case, things are more or less a repeat of the positive matrix
% variable case in sosposmatrvar.m with P replaced by LLL1 and Z with ZZZth
% Note, however that because this rearranges the order of the elements of
% ZZZth, we must alter all the manipulations or the result will be invalid.

%%%%%%%%%%%%%%%%%%%%%%%%%
% Construct M
LLL1=LLL(1:nBZ1,1:nBZ1);

avarname=[LLL1.varname; ZZZth.varname];
% For the next part, we just need the degmat part of the block-diagonal
% monomial matrix

bZdegmat=repmat(ZZZth.degmat,n,1);

[PIlist, PJlist]=find(LLL1.coefficient); % This is the list of terms in P, but PIlist and Pjlist will be relatively short. 
[PIlist2,ttt]=unique(PIlist);
PJlist2=PJlist(ttt,:);

[Prowu,Pcolu] = ind2sub([n*nZth n*nZth],PJlist2); % this returns the matrix locations for every term in P
adegmat = [LLL1.degmat(PIlist2,:) bZdegmat(Prowu,:)+bZdegmat(Pcolu,:)];


% but now we have to modify the coefficient matrix. We can group this by
% blocks. For a 2x2 matrix, all the coefficients from block 1 will be [1 0 0 0], all the
% coefficients from block 2 will be [0 1 0 0], all the coefficients from
% block 3 will be [0 0 0 1],

% Positions in Pjlist from 1:nZ get reassigned to 1, nZ+1:2nZ -> 2, etc
[row,col] = ind2sub([n*nZth n*nZth],PJlist);
newrow=ceil(row/nZth);
newcol=ceil(col/nZth);
newidx=sub2ind([n n],newrow, newcol);

coeff=sparse(PIlist,newidx,1);
amatdim=[n n];
M=polynomial(coeff,adegmat,avarname,amatdim);

%%%%%%%%%%%%%%%%%%%%%%%%%
% Construct A
% A is slightly different because the monomials of the left and the right
% are different, with different sizes. Thus the positions of the
% coefficients will also be different
% A=bZ1th.'*LLL(1:nBZ1,(nBZ1+1):(nBZ1+nBZ2))*bZ2thksi;
LLL2=LLL(1:nBZ1,(nBZ1+1):(nBZ1+nBZ2));

avarname=[LLL2.varname; ZZZthksi.varname ];
% For the next part, we just need the degmat part of the block-diagonal
% monomial matrix
if strcmp(var1.varname{1},ZZZthksi.varname{1}); % var1 is in the first position
        bZdegmat1=repmat([ZZZth.degmat zeros(nZth,1)],n,1);
else
        bZdegmat1=repmat([zeros(nZth,1) ZZZth.degmat],n,1);
end
    bZdegmat2=repmat(ZZZthksi.degmat,n,1);

[PIlist, PJlist]=find(LLL2.coefficient); % This is the list of terms in P, but PIlist and Pjlist will be relatively short. 
[Prowu,Pcolu] = ind2sub([n*nZth n*nZthksi],PJlist); % this returns the matrix locations for every term in P
adegmat = [LLL2.degmat(PIlist,:) bZdegmat1(Prowu,:)+bZdegmat2(Pcolu,:)];

    
% 
% but now we have to modify the coefficient matrix. We can group this by
% blocks. For a 2x2 matrix, all the coefficients from block 1 will be [1 0 0 0], all the
% coefficients from block 2 will be [0 1 0 0], all the coefficients from
% block 3 will be [0 0 0 1],

% Positions in Pjlist from 1:nZ get reassigned to 1, nZ+1:2nZ -> 2, etc
[row,col] = ind2sub([n*nZth n*nZthksi],PJlist);
newrow=ceil(row/nZth);
newcol=ceil(col/nZthksi);
newidx=sub2ind([n n],newrow, newcol);%(newcol-1)*n+newrow;

nPIlist=(1:length(PIlist))';
coeff=sparse(nPIlist,newidx,1);
amatdim=[n n];
A=polynomial(coeff,adegmat,avarname,amatdim);
%A(1,1)

%%%%%%%%%%%%%%%%%%%%%%%%%
% Construct B
% We use the fact that
% B=bZ2ksith.'*LLL((nBZ1+1):(nBZ1+nBZ2),1:nBZ1)*bZ1ksi
% =(bZ1th.'*LLL(1:nBZ1,(nBZ1+1):(nBZ1+nBZ2))*bZ2thksi)' with the variables
% th and ksi swapped
% Swapping the variables only requires rearranging the degmat slightly

bdegmat = [adegmat(:,1:(size(adegmat,2)-2)) adegmat(:,size(adegmat,2)) adegmat(:,size(adegmat,2)-1)];
B=polynomial(coeff,bdegmat,avarname,amatdim);
B=B.';
%%%%%%%%%%%%%%%%%%%%%%%%%
% Construct CA
%CA=bZ2omth.'*LLL((nBZ1+1):(nBZ1+nBZ2),(nBZ1+1):(nBZ1+nBZ2))*bZ2omksi;
LLL4=LLL((nBZ1+1):(nBZ1+nBZ2),(nBZ1+1):(nBZ1+nBZ2));
% first, lets construct Zomth and Zomksi
avarname=[LLL4.varname; var1.varname; var2.varname; dum.varname{1}];

 if strcmp(var1.varname{1},ZZZthksi.varname{1}) % th is in the first position
         bZdegmat1=repmat([ZZZthksi.degmat(:,2) zeros(nZthksi,1)  ZZZthksi.degmat(:,1)],n,1); %bZomth
         bZdegmat2=repmat([zeros(nZthksi,1) ZZZthksi.degmat(:,2)  ZZZthksi.degmat(:,1)],n,1); %bZomksi
 else
     bZdegmat1=repmat([ZZZthksi.degmat(:,1) zeros(nZthksi,1)  ZZZthksi.degmat(:,2)],n,1); %bZomth
         bZdegmat2=repmat([zeros(nZthksi,1) ZZZthksi.degmat(:,1)  ZZZthksi.degmat(:,2)],n,1); %bZomksi
 end

[PIlist, PJlist]=find(LLL4.coefficient); % This is the list of terms in P, but PIlist and Pjlist will be relatively short. 
[Prowu,Pcolu] = ind2sub([n*nZthksi n*nZthksi],PJlist); % this returns the matrix locations for every term in P
adegmat = [LLL4.degmat(PIlist,:) bZdegmat1(Prowu,:)+bZdegmat2(Pcolu,:)];
% but now we have to modify the coefficient matrix. We can group this by
% blocks. For a 2x2 matrix, all the coefficients from block 1 will be [1 0 0 0], all the
% coefficients from block 2 will be [0 1 0 0], all the coefficients from
% block 3 will be [0 0 0 1],

newrow=ceil(Prowu/nZthksi);
newcol=ceil(Pcolu/nZthksi);

% [row,col] = ind2sub([n*nZthksi n*nZthksi],PJlist);
% newrow=ceil(row/nZthksi);
% newcol=ceil(col/nZthksi);
newidx=sub2ind([n n],newrow, newcol);%(newcol-1)*n+newrow;
nPIlist=(1:length(PIlist))';
coeff=sparse(nPIlist,newidx,1);
amatdim=[n n];
CA=polynomial(coeff,adegmat,avarname,amatdim);
C=int(CA,dum,I(1),I(2));
N=A+B+C;
%clear ss;
end
